#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <wiringPiI2C.h>
#include "uss.h"

// 測定の時間間隔[ms]
static const int t_uss = 50;

// 測定範囲の下限・上限[cm]
static const int uss_min =  16;
//static const int uss_max = 600;
//static const int uss_max = 100;

// センサからMIRS中心までの距離[cm]
static const int dist_center = 10;

static int fd_l, fd_r;

static void *thread_code(void*);
static pthread_t task_id;

static long uss_l, uss_r;

void *thread_code(void *t){
        // 実行周期[ms]
        const int cycle = 10 ; 

        while(1){
                uss_l = uss_get_l();
                usleep(cycle * 1000);
                uss_r = uss_get_r();
                usleep(cycle * 1000);
        }

        return 0;
}

int uss_create(){

	int policy;
        struct sched_param param;

        // スレッドの生成
        if(pthread_create(&task_id, NULL, thread_code, NULL) != 0){
                printf("failed to create uss thread\n");
                return -1;
        }

        // スレッドパラメータの確認
        pthread_getschedparam(task_id, &policy, &param);
        printf("uss thread created (plicy = %s, priority = %d)\n",
                ((policy == SCHED_FIFO) ? "SCHED_FIFO" : (policy == SCHED_RR) ? "SCHED_RR" : (policy == SCHED_OTHER) ? "SCHED_OTHER" : "???"),
                param.sched_priority);

        return 0;
}


void uss_get_val(int *usd_l, int *usd_r){

	*usd_l = uss_l;
	*usd_r = uss_r;

}

int uss_open_l(){
	fd_l = wiringPiI2CSetup(0x70);
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_l, 0x00) == 0x06){
		printf("USS_L opened\n");
		return 0;
	}else{
		printf("failed to open USS_L\n");
		return -1;
	}
}

int uss_open_r(){
	fd_r = wiringPiI2CSetup(0x71);
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_r, 0x00) == 0x06){
		printf("USS_R opened\n");
		return 0;
	}else{
		printf("failed to open USS_R\n");
		return -1;
	}
}

long uss_get_l(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_l, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_l, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_l, 0x02);
	val = lowbyte + highbyte * 256;

	#ifdef USS_DEBUG
		printf( "val_l = %d\n", (int)val);
	#endif
	
	//if(val < uss_min || val > uss_max){
	if(val < uss_min ){
		return -1;
	//}else if(val > uss_max){
	//	return 0;
	}else{
		return val + dist_center;
	}
}

long uss_get_r(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_r, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_r, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_r, 0x02);
	val = lowbyte + highbyte * 256;

	#ifdef USS_DEBUG
		printf( "val_r = %d\n", (int)val);
	#endif
	
	//if(val < uss_min || val > uss_max){
	if(val < uss_min ){
		return -1;
	//}else if(val > uss_max){
	//	return 0;
	}else{
		return val + dist_center;
	}
}
